#include "animated_polygon.h"
using namespace std;


namespace ist {

BaseAnimatedPolygon::BaseAnimatedPolygon() :
	num_vertex(0)
{}

BaseAnimatedPolygon::~BaseAnimatedPolygon()
{}

void BaseAnimatedPolygon::draw(const string& clipname, float t) const
{}

void BaseAnimatedPolygon::setup()
{}


struct AVTXHeader {
	unsigned int version_number;
	unsigned int index_count;
	unsigned int vertex_count;
	unsigned int vertex_clip_count;
};

struct AVTXClipHeader {
	unsigned int frame_count;
	unsigned int label_length;
};



bool BaseAnimatedPolygon::load(const string& filename) {
	clear();

	if(strncmp(&filename[filename.size()]-4, ".avx", 5)!=0)
		return false;

	FILE *in = fopen(filename.c_str(), "rb");
	if(in==0) return false;

	AVTXHeader header;
	fread(&header.version_number, sizeof(int), 1, in);
	fread(&header.index_count, sizeof(int), 1, in);
	fread(&header.vertex_count, sizeof(int), 1, in);
	fread(&header.vertex_clip_count, sizeof(int), 1, in);

	if(header.version_number!=100)
		return false;


	idx_array.resize(header.index_count);
	tex_array.resize(header.index_count);
	num_vertex = header.vertex_count;


	for(unsigned int i=0; i<idx_array.size(); ++i)
		fread(&idx_array[i], sizeof(int), 1, in);

	for(unsigned int i=0; i<tex_array.size(); ++i) {
		fread(&tex_array[i].x, sizeof(float), 1, in);
		fread(&tex_array[i].y, sizeof(float), 1, in);
	}

	for(unsigned int i=0; i<header.vertex_clip_count; ++i) {
		AVTXClipHeader clip_header;
		vector<char> temp_string(256, '\0');
		fread(&clip_header.frame_count, sizeof(int), 1, in);
		fread(&clip_header.label_length, sizeof(int), 1, in);
		fread(&temp_string[0], sizeof(char), clip_header.label_length, in);

		vertex_sequence& seq = vtx_clip[&temp_string[0]];

		for(unsigned int j=0; j<clip_header.frame_count; ++j) {
			float t;
			fread(&t, sizeof(float), 1, in);

			vertex_cont& vcont = seq[t];
			vcont.resize(header.vertex_count);

			for(unsigned int k=0; k<vcont.size(); ++k) {
				fread(&vcont[k].x, sizeof(float), 1, in);
				fread(&vcont[k].y, sizeof(float), 1, in);
				fread(&vcont[k].z, sizeof(float), 1, in);
			}
		}
	}

	fclose(in);

	setup();

	return true;
}

bool BaseAnimatedPolygon::save(const string& filename) {
	FILE *out = fopen(filename.c_str(), "wb");
	if(out==0) return false;

	AVTXHeader header={
		100,
		idx_array.size(),
		num_vertex,
		vtx_clip.size()
	};

	fwrite(&header.version_number, sizeof(int), 1, out);
	fwrite(&header.index_count, sizeof(int), 1, out);
	fwrite(&header.vertex_count, sizeof(int), 1, out);
	fwrite(&header.vertex_clip_count, sizeof(int), 1, out);

	for(unsigned int i=0; i<idx_array.size(); ++i)
		fwrite(&idx_array[i], sizeof(int), 1, out);

	for(unsigned int i=0; i<tex_array.size(); ++i) {
		fwrite(&tex_array[i].x, sizeof(float), 1, out);
		fwrite(&tex_array[i].y, sizeof(float), 1, out);
	}

	for(vertex_clip::iterator p=vtx_clip.begin(); p!=vtx_clip.end(); ++p) {
		vertex_sequence& seq = p->second;
		AVTXClipHeader clip_header={
			seq.size(),
			p->first.size(),
		};

		fwrite(&clip_header.frame_count, sizeof(int), 1, out);
		fwrite(&clip_header.label_length, sizeof(int), 1, out);
		fwrite(p->first.c_str(), sizeof(char), p->first.size(), out);

		for(vertex_sequence::iterator q=seq.begin(); q!=seq.end(); ++q) {
			fwrite(&q->first, sizeof(float), 1, out);

			vertex_cont& vcont = q->second;
			for(unsigned int i=0; i<vcont.size(); ++i) {
				fwrite(&vcont[i].x, sizeof(float), 1, out);
				fwrite(&vcont[i].y, sizeof(float), 1, out);
				fwrite(&vcont[i].z, sizeof(float), 1, out);
			}
		}
	}

	fclose(out);

	return true;
}

void BaseAnimatedPolygon::clear() {
	vtx_clip.clear();
	tex_array.clear();
	idx_array.clear();
	num_vertex=0;
}

} // ist
