#ifndef IST_ANIMATED_POLYGON
#define IST_ANIMATED_POLYGON

#include <algorithm>
#include <functional>
#include <string>
#include <vector>
#include <map>
#include "ist/ist_math.h"

namespace ist {

class BaseAnimatedPolygon {
public:
	typedef std::vector<vector2> texcoord_cont;
	typedef std::vector<int>     index_cont;
	typedef std::vector<vector3> vertex_cont;

	class vertex_sequence : public std::map<float, vertex_cont>
	{
	public:
		typedef std::pair<const_iterator, const_iterator> range_t;

		float begin_frame() const {
			if(empty()) return 0;
			return begin()->first;
		}

		float end_frame() const {
			if(empty()) return 0;
			return (--end())->first;
		}

		bool change_frame(float before, float after) {
			iterator p = find(before);
			if(p==end())
				return false;

			(*this)[after] = p->second;
			erase(p);
			return true;
		}

		range_t range(float f) const {
			return equal_range(f);
		}
	};

	typedef std::map<std::string, vertex_sequence> vertex_clip;


	BaseAnimatedPolygon();
	virtual ~BaseAnimatedPolygon();
	virtual void draw(const std::string& clipname, float t) const;
	virtual void setup();
	bool load(const std::string& filename);
	bool save(const std::string& filename);
	void clear();

	float beginFrame(const std::string& clipname) const {
		vertex_clip::const_iterator p = vtx_clip.find(clipname);
		if(p==vtx_clip.end())
			return 0;
		return p->second.begin_frame();
	}

	float endFrame(const std::string& clipname) const {
		vertex_clip::const_iterator p = vtx_clip.find(clipname);
		if(p==vtx_clip.end())
			return 0;
		return p->second.end_frame();
	}

	bool changeFrame(const std::string& clipname, float before, float after) {
		vertex_clip::iterator p = vtx_clip.find(clipname);
		if(p==vtx_clip.end())
			return false;
		return p->second.change_frame(before, after);
	}


	vertex_clip   vtx_clip;
	texcoord_cont tex_array;
	index_cont    idx_array;
	unsigned int  num_vertex;
};


} // ist

#endif
