#include "gl_animated_polygon.h"
#include "frame_preview.h"


class ModelElement : public ist::BasePolygonModel
{
public:
	void setup() {}
};

class ModelEntry
{
public:
	ModelEntry() : frame(0) {}

	wxString longname;
	wxString name;
	wxString path;
	int frame;

	void resetLongname() {
		wxString temp;
		temp.Printf("%d - %s", frame, name.c_str());
		longname = temp;
	}

	bool operator==(const wxString& lhs) const { return lhs==longname; }
	bool operator<(const ModelEntry& rhs) const { return frame < rhs.frame; }
};




namespace {
	typedef list<ModelEntry> ModelEntryCont;
	typedef map<wxString, ModelEntryCont> ModelEntryMap;

	gl::AnimatedPolygon a_model;
	ModelEntryMap model_entry;
	wxString texture_path;
}

wxString GetClipName()
{
	return GetPreviewFrame()->clip_choice->GetStringSelection();
}

const float GetFrame()
{
	return (float)GetPreviewFrame()->timeline_slider->GetValue()/100;
}

void OutputText(const wxString& message)
{
	GetPreviewFrame()->log_window->SetValue(message);
}

void ResetEntry()
{
	model_entry.clear();

	for(gl::AnimatedPolygon::vertex_clip::iterator q=a_model.vtx_clip.begin(); q!=a_model.vtx_clip.end(); ++q) {
		gl::AnimatedPolygon::vertex_sequence &seq = q->second;
		ModelEntryCont &entry = model_entry[q->first.c_str()];

		for(gl::AnimatedPolygon::vertex_sequence::iterator p=seq.begin(); p!=seq.end(); ++p) {
			ModelEntry temp;
			temp.name = q->first.c_str();
			temp.frame = p->first;
			temp.resetLongname();
			entry.push_back(temp);
		}
	}
}

void GeneratePartAVTX(const wxString& clip_name)
{
	a_model.vtx_clip[clip_name.c_str()].clear();

	ModelElement model;

	for(ModelEntryCont::iterator p=model_entry[clip_name].begin(); p!=model_entry[clip_name].end(); ++p) {
		if(!model.load(p->path.c_str()) || model.empty())
			continue;

		BasePolygonChunc &chunc = *model.front();

		if(a_model.num_vertex!=0) {
			if(a_model.num_vertex!=chunc.vtx_array.size())
				continue;
		}
		else {
			a_model.idx_array = chunc.idx_array;
			a_model.tex_array = chunc.tex_array;
			a_model.num_vertex = chunc.vtx_array.size();
		}

		gl::AnimatedPolygon::vertex_cont &vtx = a_model.vtx_clip[clip_name.c_str()][p->frame];
		vtx.clear();
		vtx.resize(a_model.num_vertex);
		for(unsigned int i=0; i<a_model.num_vertex; ++i) {
			vtx[i] = model.front()->vtx_array[i].to_vector3();
		}
	}

	if(a_model.vertex_array.empty())
		a_model.setup();
}

void GenerateAVTX()
{
	if(model_entry.empty())
		return;

	for(ModelEntryMap::iterator q=model_entry.begin(); q!=model_entry.end(); ++q)
		GeneratePartAVTX(q->first.c_str());
}





class PreviewCanvas : public wxGLCanvas
{
friend class PreviewFrame;
public:
	PreviewCanvas(wxWindow *parent, const wxPoint& = wxDefaultPosition, const wxSize& = wxDefaultSize);
	PreviewCanvas(wxWindow *parent, const PreviewCanvas& other,
		const wxPoint& = wxDefaultPosition, const wxSize& = wxDefaultSize);
	void Init();
	void Render();
	void OnPaint(wxPaintEvent& event);
	void OnSize(wxSizeEvent& event);
	void OnMouse(wxMouseEvent& event);

private:
	int display_mode;
	gl::Camera camera;
	gl::Light light;
	vector2 mouse_pos, pre_mouse_pos;
	vector4 pre_camera_pos;

	DECLARE_EVENT_TABLE()
};





enum
{
	MENU_NEW_PROJECT = 100,
	MENU_SAVE_PROJECT,
	MENU_LOAD_PROJECT,
	MENU_SAVE_MODEL,
	MENU_LOAD_MODEL,
	MENU_LOAD_TEXTURE,
	MENU_INFOMATION,

	SLIDER_TIMELINE,
	CHOICE_CLIP,
	LISTBOX_MODELS,
	TEXTCTRL_TIMELINE,
	BUTTON_NEW_CLIP,
	BUTTON_DELETE_CLIP,
	BUTTON_LOAD,
	BUTTON_SORT,
	BUTTON_DELETE,
	BUTTON_INVERTV,
	BUTTON_RELOAD,
	TEXTCTRL_ENTRY_NAME,
	TEXTCTRL_ENTRY_FRAME,
};


BEGIN_EVENT_TABLE( PreviewFrame, wxFrame )
	EVT_MENU( MENU_SAVE_PROJECT, PreviewFrame::MenuSaveProject )
	EVT_MENU( MENU_LOAD_PROJECT, PreviewFrame::MenuLoadProject )
	EVT_MENU( MENU_SAVE_MODEL, PreviewFrame::MenuSaveModel )
	EVT_MENU( MENU_LOAD_MODEL, PreviewFrame::MenuLoadModel )
	EVT_MENU( MENU_LOAD_TEXTURE, PreviewFrame::MenuLoadTexture )
	EVT_MENU( MENU_INFOMATION, PreviewFrame::MenuInfomation )

	EVT_PAINT( PreviewFrame::OnPaint )
	EVT_COMMAND_SCROLL( SLIDER_TIMELINE, PreviewFrame::SliderChanged )
	EVT_CHOICE( CHOICE_CLIP, PreviewFrame::ChoiceClip )
	EVT_LISTBOX( LISTBOX_MODELS, PreviewFrame::EntryClicked )
	EVT_BUTTON( BUTTON_NEW_CLIP, PreviewFrame::NewClip )
	EVT_BUTTON( BUTTON_DELETE_CLIP, PreviewFrame::DeleteClip )
	EVT_BUTTON( BUTTON_LOAD, PreviewFrame::AddEntry )
	EVT_BUTTON( BUTTON_DELETE, PreviewFrame::RemoveEntry )
	EVT_BUTTON( BUTTON_RELOAD, PreviewFrame::ReloadModel )
	EVT_BUTTON( BUTTON_INVERTV, PreviewFrame::InvertVCoord )
	EVT_TEXT_ENTER( TEXTCTRL_ENTRY_NAME, PreviewFrame::ChangeEntryName )
	EVT_TEXT_ENTER( TEXTCTRL_ENTRY_FRAME, PreviewFrame::ChangeEntryFrame )
END_EVENT_TABLE()


PreviewFrame::PreviewFrame(wxWindow *parent) :
	wxFrame(parent, -1, wxT("preview"), wxDefaultPosition, wxSize(800, 633))
{
	wxMenu *menu_file = new wxMenu();
	menu_file->Append(MENU_SAVE_PROJECT, wxT("Save Project"));
	menu_file->Append(MENU_LOAD_PROJECT, wxT("Load Project"));
	menu_file->AppendSeparator();
	menu_file->Append(MENU_SAVE_MODEL, wxT("Save Model"));
	menu_file->Append(MENU_LOAD_MODEL, wxT("Load Model"));
	menu_file->AppendSeparator();
	menu_file->Append(MENU_LOAD_TEXTURE, wxT("Load Texture"));
	menu_file->AppendSeparator();
	menu_file->Append(MENU_INFOMATION, wxT("Info"));

	wxMenuBar *menu_bar = new wxMenuBar();
	menu_bar->Append(menu_file, wxT("&File"));
	SetMenuBar(menu_bar);


	wxBoxSizer *top_sizer = new wxBoxSizer(wxHORIZONTAL);

	{
		wxPanel *slider_panel = new wxPanel( this, -1, wxPoint(0, 0), wxSize(800, 33) );
		wxBoxSizer *slider_sizer = new wxBoxSizer(wxHORIZONTAL);
		timeline_slider = new wxSlider( slider_panel, SLIDER_TIMELINE, 0, 0, 1000 );
		timeline_text = new wxTextCtrl( slider_panel, TEXTCTRL_TIMELINE, wxT("0"), wxDefaultPosition, wxSize(40, -1) );
		slider_sizer->Add( timeline_slider, 1, wxEXPAND );
		slider_sizer->Add( timeline_text, 0, wxALIGN_RIGHT );
		slider_panel->SetAutoLayout( TRUE );
		slider_panel->SetSizer( slider_sizer );

		canvas = new PreviewCanvas(this, wxPoint(0, 0), wxSize(800, 600));

		wxBoxSizer *left_sizer = new wxBoxSizer(wxVERTICAL);
		left_sizer->Add( canvas, 1, wxEXPAND );
		left_sizer->Add( slider_panel, 0, wxEXPAND );

		top_sizer->Add( left_sizer, 1, wxEXPAND);
	}

	{
		right_panel = new wxPanel( this, -1, wxPoint(0, 0), wxSize(200, 30), wxSUNKEN_BORDER );
		wxBoxSizer *right_sizer = new wxBoxSizer(wxVERTICAL);

		{
			wxBoxSizer *sizer = new wxStaticBoxSizer( new wxStaticBox(right_panel, -1, wxT("animation clip")), wxVERTICAL );

			clip_choice = new wxChoice( right_panel, CHOICE_CLIP, wxDefaultPosition, wxDefaultSize);
			model_entry[wxT("default")];
			RefreshClipChoice();
			clip_choice->SetStringSelection(wxT("default"));

			wxButton *clip_button1 = new wxButton( right_panel, BUTTON_NEW_CLIP, wxT("new clip") );
			wxButton *clip_button2 = new wxButton( right_panel, BUTTON_DELETE_CLIP, wxT("delete clip") );
			wxBoxSizer *clip_button_sizer = new wxBoxSizer(wxHORIZONTAL);
			clip_button_sizer->Add( clip_button1, 0, wxEXPAND | wxALL, 5 );
			clip_button_sizer->Add( clip_button2, 0, wxEXPAND | wxALL, 5 );

			sizer->Add( clip_choice, 0, wxEXPAND | wxALL, 5 );
			sizer->Add( clip_button_sizer, 0, wxEXPAND );
			right_sizer->Add( sizer, 0, wxEXPAND | wxALL, 5 );
			right_sizer->Add( 10, 10 );
		}

		{
			wxBoxSizer *sizer = new wxStaticBoxSizer( new wxStaticBox(right_panel, -1, wxT("model entry")), wxVERTICAL );

			model_listbox = new wxListBox( right_panel, LISTBOX_MODELS, wxDefaultPosition, wxSize(10, 120));

			wxBoxSizer *button_sizer1 = new wxBoxSizer(wxHORIZONTAL);
			button_sizer1->Add( new wxButton( right_panel, BUTTON_LOAD, wxT("import model") ), 0, wxEXPAND|wxALL, 5 );
			button_sizer1->Add( new wxButton( right_panel, BUTTON_DELETE, wxT("delete entry") ), 0, wxEXPAND|wxALL, 5 );

			wxBoxSizer *button_sizer2 = new wxBoxSizer(wxHORIZONTAL);
			button_sizer2->Add( new wxButton( right_panel, BUTTON_RELOAD, wxT("reload model") ), 0, wxEXPAND|wxALL, 5 );
			button_sizer2->Add( new wxButton( right_panel, BUTTON_INVERTV, wxT("invert v") ), 0, wxEXPAND|wxALL, 5 );

			sizer->Add( model_listbox, 0, wxEXPAND | wxALL, 5 );
			sizer->Add( button_sizer1, 0, wxEXPAND );
			sizer->Add( button_sizer2, 0, wxEXPAND );
			right_sizer->Add( sizer, 0, wxEXPAND | wxALL, 5 );
			right_sizer->Add( 10, 10 );
		}

		{
			wxBoxSizer *sizer = new wxStaticBoxSizer( new wxStaticBox(right_panel, -1, wxT("entry property")), wxVERTICAL );

			entry_name  = new wxTextCtrl( right_panel, TEXTCTRL_ENTRY_NAME, wxT(""));
			entry_frame = new wxTextCtrl( right_panel, TEXTCTRL_ENTRY_FRAME, wxT(""));
			entry_path  = new wxTextCtrl( right_panel, TEXTCTRL_ENTRY_FRAME, wxT(""));

			wxBoxSizer *text_sizer1 = new wxBoxSizer(wxHORIZONTAL);
			text_sizer1->Add( new wxStaticText(right_panel, -1, wxT("name:")), 0, wxALIGN_CENTRE_VERTICAL);
			text_sizer1->Add( entry_name, 0);

			wxBoxSizer *text_sizer2 = new wxBoxSizer(wxHORIZONTAL);
			text_sizer2->Add( new wxStaticText(right_panel, -1, wxT("frame:")), 0, wxALIGN_CENTRE_VERTICAL);
			text_sizer2->Add( entry_frame, 0);

			wxBoxSizer *text_sizer3 = new wxBoxSizer(wxHORIZONTAL);
			text_sizer3->Add( new wxStaticText(right_panel, -1, wxT("path:")), 0, wxALIGN_CENTRE_VERTICAL);
			text_sizer3->Add( entry_path, 0);

			sizer->Add( text_sizer1, 0, wxEXPAND | wxALL, 2 );
			sizer->Add( text_sizer2, 0, wxEXPAND | wxALL, 2 );
			sizer->Add( text_sizer3, 0, wxEXPAND | wxALL, 2 );
			right_sizer->Add( sizer, 0, wxEXPAND | wxALL, 5 );
			right_sizer->Add( 10, 10 );
		}

		/*
		{
			log_window = new wxTextCtrl( right_panel, -1, wxT(""),
				wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxTE_READONLY);
			right_sizer->Add( log_window, 0, wxEXPAND | wxALL, 5 );
		}
		*/

		right_panel->SetAutoLayout( TRUE );
		right_panel->SetSizer( right_sizer );
		top_sizer->Add( right_panel, 0, wxEXPAND);
	}

	SetAutoLayout( TRUE );
	SetSizer( top_sizer );


	CreateStatusBar();
}

void PreviewFrame::OnPaint(wxPaintEvent& event)
{
	wxFrame::OnPaint(event);
	wxPaintDC dc(this);
	canvas->Render();
	right_panel->Refresh();
}


void PreviewFrame::MenuSaveProject(wxCommandEvent&)
{
	wxFileDialog file_dlg(this, wxT("save project"), wxT(""), wxT(""), wxT("*.avp"), wxSAVE );

	if(file_dlg.ShowModal() == wxID_OK) {
		FILE *out = fopen(file_dlg.GetPath().c_str(), "w");
		if(!out)
			return;

		fprintf(out, "texture %s\n", texture_path.c_str());
		fprintf(out, "\n");

		for(ModelEntryMap::iterator p=model_entry.begin(); p!=model_entry.end(); ++p) {
			ModelEntryCont& entry = p->second;

			fprintf(out, "clip %s, %d\n", p->first.c_str(), p->second.size());

			for(ModelEntryCont::iterator q=entry.begin(); q!=entry.end(); ++q)
				fprintf(out, "entry %s, %d, %s\n", q->name.c_str(), q->frame, q->path.c_str());
			fprintf(out, "\n");
		}
	}
}

void PreviewFrame::MenuLoadProject(wxCommandEvent&)
{
	wxFileDialog file_dlg(this, wxT("load project"), wxT(""), wxT(""), wxT("*.avp") );

	if(file_dlg.ShowModal() == wxID_OK) {
		model_entry.clear();
		a_model.clear();

		FILE *in = fopen(file_dlg.GetPath().c_str(), "r");
		if(!in)
			return;

		char buf[1024];
		while(fgets(buf, 1024, in)) {
			char name[1024];
			int num_entry;

			if(sscanf(buf, "texture %[^\n]", name)==1) {
				texture_path = name;
				GetTexture().load( texture_path.c_str() );
			}
			else if(sscanf(buf, "clip %[^,], %d", name, &num_entry)==2) {
				for(int i=0; i<num_entry; ++i) {
					char entry_name[1024];
					char entry_path[1024];
					int f=0;

					fgets(buf, 1024, in);
					if(sscanf(buf, "entry %[^,], %d, %[^\n]", entry_name, &f, entry_path)==3) {
						ModelEntry temp;
						temp.name = wxT(entry_name);
						temp.frame = f;
						temp.path = wxT(entry_path);
						temp.resetLongname();

						model_entry[name].push_back(temp);
					}
				}

				GeneratePartAVTX(name);
			}
		}

		RefreshClipChoice();
		clip_choice->SetStringSelection(wxT("default"));
		RefreshModelList();
	}
}


void PreviewFrame::MenuSaveModel(wxCommandEvent&)
{
	wxFileDialog file_dlg(this, wxT("save model"), wxT(""), wxT(""), wxT("*.avx"), wxSAVE );

	if(file_dlg.ShowModal() == wxID_OK) {
		a_model.save( file_dlg.GetPath().c_str() );
	}
}

void PreviewFrame::MenuLoadModel(wxCommandEvent& WXUNUSED(event))
{
	wxFileDialog file_dlg(this, wxT("open model"), wxT(""), wxT(""), wxT("*.avx"));

	if(file_dlg.ShowModal() == wxID_OK) {
		model_entry.clear();
		a_model.clear();

		canvas->SetCurrent();
		bool result = a_model.load( file_dlg.GetPath().c_str() );
		if(result) {
			ResetEntry();
			RefreshClipChoice();
			clip_choice->SetStringSelection(wxT("default"));
			RefreshModelList();

			SetStatusText( wxT("load success") );
		}
		else
			SetStatusText( wxT("load failed") );
	}
}

void PreviewFrame::MenuLoadTexture(wxCommandEvent& WXUNUSED(event))
{
	wxFileDialog file_dlg(this, wxT("open texture"), wxT(""), wxT(""),
		wxT("*.bmp;*.tga") );

	if(file_dlg.ShowModal() == wxID_OK) {
		canvas->SetCurrent();

		texture_path = file_dlg.GetPath();
		bool result = GetTexture().load( texture_path.c_str() );
		if(result)
			SetStatusText( wxT("load success") );
		else
			SetStatusText( wxT("load failed") );

		Refresh();
	}
}

void PreviewFrame::MenuInfomation(wxCommandEvent& WXUNUSED(event))
{
	wxString info;
	wxString temp;

	for(gl::AnimatedPolygon::vertex_clip::iterator p=a_model.vtx_clip.begin(); p!=a_model.vtx_clip.end(); ++p) {
		temp.Printf("%s\n%d keyframe\n\n",
			p->first.c_str(), p->second.size());
		info+=temp;
	}

	temp.Printf("\n%d vertex\n%d triangle",
		a_model.num_vertex, a_model.idx_array.size()/3);
	info+=temp;

	wxMessageDialog *dialog = new wxMessageDialog(this, info, wxT("Model Info"));
	dialog->ShowModal();
}

void PreviewFrame::SliderChanged(wxScrollEvent& event)
{
	wxString value;
	value.Printf("%.2f", GetFrame());
	timeline_text->SetValue(value);

	canvas->Render();
}


void PreviewFrame::NewClip(wxCommandEvent&)
{
	wxTextEntryDialog dialog(this, wxT("enter clip name"), wxT("new clip"));

	if(dialog.ShowModal()==wxID_OK && dialog.GetValue().Length()!=0) {
		model_entry[dialog.GetValue()];
		RefreshClipChoice();
		clip_choice->SetStringSelection(dialog.GetValue());
		RefreshModelList();
	}

	entry_name->SetValue( wxT("") );
	entry_frame->SetValue( wxT("") );
}

void PreviewFrame::DeleteClip(wxCommandEvent& WXUNUSED(event))
{
	if(GetClipName() == wxT("default"))
		return;

	wxMessageDialog dialog(this, wxT("delete ") + GetClipName() + wxT(", are you sure?"),
		wxT("delete clip"), wxOK | wxCANCEL | wxCENTRE);

	if(dialog.ShowModal() == wxID_OK) {
		model_entry.erase( model_entry.find(GetClipName()) );
		RefreshClipChoice();
		clip_choice->SetStringSelection(wxT("default"));

		entry_name->SetValue( wxT("") );
		entry_frame->SetValue( wxT("") );
		RefreshModelList();
	}
}


void PreviewFrame::AddEntry(wxCommandEvent& WXUNUSED(event))
{
	wxFileDialog file_dlg(this, wxT("open model"), wxT(""), wxT(""),
		wxT("*.obj;*.mqo"), wxOPEN | wxMULTIPLE );

	if(file_dlg.ShowModal() == wxID_OK) {
		wxArrayString name, path;
		file_dlg.GetFilenames(name);
		file_dlg.GetPaths(path);
		name.Sort();
		path.Sort();

		for(size_t i=0; i<name.GetCount(); ++i) {
			ModelEntryCont& entry = model_entry[ GetClipName() ];
			int frame;
			if(entry.empty())
				frame = 0;
			else
				frame = entry.back().frame+30;

			entry.push_back(ModelEntry());
			entry.back().name = name[i];
			entry.back().path = path[i];
			entry.back().frame = frame;
			entry.back().resetLongname();
		}

		GeneratePartAVTX(GetClipName());
		RefreshModelList();
	}
}


void PreviewFrame::RemoveEntry(wxCommandEvent& WXUNUSED(event))
{
	int pos = model_listbox->GetSelection();
	if(pos==wxNOT_FOUND)
		return;

	ModelEntryCont& entry = model_entry[ GetClipName() ];
	ModelEntryCont::iterator p = entry.begin();
	advance(p, pos);

	gl::AnimatedPolygon::vertex_sequence& seq = a_model.vtx_clip[GetClipName().c_str()];
	seq.erase( seq.find(p->frame) );
	entry.erase(p);

	RefreshModelList();
}


void PreviewFrame::EntryClicked(wxCommandEvent& event)
{
	ModelEntryCont& entry = model_entry[ GetClipName() ];
	ModelEntryCont::iterator p = find(entry.begin(), entry.end(), event.GetString());

	if(p!=entry.end()) {
		wxString temp;
		temp.Printf("%d", p->frame);

		entry_name->SetValue( p->name );
		entry_frame->SetValue( temp );
		entry_path->SetValue( p->path );
	}
}

void PreviewFrame::ChoiceClip(wxCommandEvent&)
{
	entry_name->SetValue( wxT("") );
	entry_frame->SetValue( wxT("") );

	RefreshModelList();
}

void PreviewFrame::ChangeEntryName(wxCommandEvent& event)
{
	ModelEntryCont& entry = model_entry[ GetClipName() ];
	ModelEntryCont::iterator p = find(entry.begin(), entry.end(), model_listbox->GetStringSelection());
	if(p==entry.end())
		return;

	p->name = entry_name->GetValue();
	p->resetLongname();
	RefreshModelList();
}

void PreviewFrame::ChangeEntryFrame(wxCommandEvent&)
{
	ModelEntryCont& entry = model_entry[ GetClipName() ];
	ModelEntryCont::iterator p = find(entry.begin(), entry.end(), model_listbox->GetStringSelection());
	if(p==entry.end())
		return;

	long f;
	if(entry_frame->GetValue().ToLong(&f)) {
		a_model.changeFrame(GetClipName().c_str(), p->frame, f);
		p->frame = f;
		p->resetLongname();

		entry.sort();
		RefreshModelList();
	}
}


void PreviewFrame::ReloadModel(wxCommandEvent& event)
{
	ModelElement model;

	ModelEntryCont& entry = model_entry[ GetClipName() ];
	for(ModelEntryCont::iterator p=entry.begin(); p!=entry.end(); ++p) {
		if(!model.load(p->path.c_str()) || model.empty())
			continue;

		gl::AnimatedPolygon::vertex_cont &vtx = a_model.vtx_clip[GetClipName().c_str()][p->frame];
		vtx.clear();
		vtx.resize(a_model.num_vertex);
		for(unsigned int i=0; i<a_model.num_vertex; ++i) {
			vtx[i] = model.front()->vtx_array[i].to_vector3();
		}
	}

	Refresh();
}

void PreviewFrame::InvertVCoord(wxCommandEvent& event)
{
	for(unsigned int i=0; i<a_model.tex_array.size(); ++i) {
		a_model.tex_array[i].y = 1.0f-a_model.tex_array[i].y;
	}
	a_model.setup();

	Refresh();
}



void PreviewFrame::RefreshTimeSlider()
{
	timeline_slider->SetRange(
		a_model.beginFrame(GetClipName().c_str())*100.0f,
		max<float>(a_model.endFrame(GetClipName().c_str())*100.0f, 10000.0f)
	);
}

void PreviewFrame::RefreshClipChoice()
{
	clip_choice->Clear();

	model_entry[wxT("default")];
	for(ModelEntryMap::iterator p=model_entry.begin(); p!=model_entry.end(); ++p) {
		clip_choice->Append(p->first);
	}
}

void PreviewFrame::RefreshModelList()
{
	model_listbox->Clear();

	ModelEntryCont& entry = model_entry[ GetClipName() ];
	for(ModelEntryCont::iterator p=entry.begin(); p!=entry.end(); ++p) {
		model_listbox->Append(p->longname);
	}
	RefreshTimeSlider();
	Refresh();
}







BEGIN_EVENT_TABLE( PreviewCanvas, wxGLCanvas )
	EVT_SIZE( PreviewCanvas::OnSize )
	EVT_PAINT( PreviewCanvas::OnPaint )
	EVT_MOUSE_EVENTS( PreviewCanvas::OnMouse )
END_EVENT_TABLE()

PreviewCanvas::PreviewCanvas(wxWindow *parent, const wxPoint& pos, const wxSize& size) :
	wxGLCanvas(parent, -1, pos, size)
{
	Init();
}

PreviewCanvas::PreviewCanvas(wxWindow *parent, const PreviewCanvas& other, const wxPoint& pos, const wxSize& size) :
	wxGLCanvas(parent, other.GetContext(), -1, pos, size )
{
	Init();
}


void PreviewCanvas::Init()
{
	display_mode = 0;
	camera.position = vector4(0, 360, 1);
	pre_camera_pos = camera.position;
	light.position = vector4(200, 300, -200);
	camera.target = vector4(0, 0, 0);

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glShadeModel( GL_SMOOTH );
	glCullFace( GL_BACK );
	glFrontFace( GL_CCW );
	glEnable( GL_CULL_FACE );
}

void PreviewCanvas::OnPaint(wxPaintEvent& WXUNUSED(event))
{
	Render();
}

void PreviewCanvas::OnSize(wxSizeEvent& event)
{
	wxGLCanvas::OnSize(event);

	int w, h;
	GetClientSize(&w, &h);
	SetCurrent();
	glViewport(0, 0, w, h);

	Render();
}

void PreviewCanvas::OnMouse(wxMouseEvent& event)
{
	if(!event.ButtonDown() && !event.Moving() && !event.Dragging())
		return;


	bool redraw_flag = false;

	mouse_pos.x = event.GetX();
	mouse_pos.y = event.GetY();

	vector2 progress = mouse_pos - pre_mouse_pos;

	if(event.LeftDown()) {
		pre_mouse_pos = mouse_pos;
		pre_camera_pos= camera.position;
	}
	else if(event.LeftIsDown()) {
		static const matrix33 turn90degree = matrix33().rotateY(90);
		float rotate_speed = 15;
		float ang_xz = progress.y*rotate_speed*radian;
		float ang_y = -progress.x*rotate_speed*radian;
		vector3 axis = (turn90degree*vector3(camera.position.x, 0, camera.position.z)).normalize();

		matrix33 camera_rot;
		camera_rot.rotateA(axis, ang_xz);
		camera_rot.rotateY(ang_y);

		camera.position = camera_rot*camera.position;
		redraw_flag = true;
	}

	if(event.MiddleDown()) {
	}
	else if(event.MiddleIsDown()) {
	}

	if(event.RightIsDown()) {
		camera.position *= (1+(progress.x-progress.y)*0.003);
		redraw_flag = true;
	}

	pre_mouse_pos = mouse_pos;
	pre_camera_pos= camera.position;

	if(redraw_flag)
		Render();
}


void PreviewCanvas::Render()
{
	wxPaintDC dc(this);
	SetCurrent();


	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();

	int w, h;
	GetClientSize(&w, &h);

	gluPerspective( 60.0, static_cast<float>(w)/static_cast<float>(h), 1.0, 1024.0 );
	glMatrixMode( GL_MODELVIEW );

	camera.look();

	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	GetTexture().assign();
	{
		a_model.draw(GetClipName().c_str(), GetFrame());
	}

	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_DEPTH_TEST);


	SwapBuffers();
}
