#include <algorithm>
#include "gl_animated_polygon.h"
using namespace std;

namespace ist {
namespace gl {


void AnimatedPolygon::setup()
{
	vertex_array.resize(idx_array.size()*3 + tex_array.size()*2);

	for(unsigned int i=0; i<tex_array.size(); ++i) {
		vertex_array[idx_array.size()*3 + i*2 + 0] = tex_array[i].x;
		vertex_array[idx_array.size()*3 + i*2 + 1] = tex_array[i].y;
	}
}


void AnimatedPolygon::draw(const std::string& clipname, float t) const
{
	vertex_clip::const_iterator p = vtx_clip.find(clipname);
	if(p==vtx_clip.end() || p->second.empty())
		return;

	static vertex_cont t_vertex;
	t_vertex.resize(num_vertex);

	const vertex_sequence& seq = p->second;
	if(t<=seq.begin_frame()) {
		t_vertex = seq.begin()->second;
	}
	else if(t >= seq.end_frame()) {
		t_vertex = (--seq.end())->second;
	}
	else {
		vertex_sequence::range_t range = seq.equal_range(t);
		if(range.first->first==t) {
			t_vertex = range.first->second;
		}
		else {
			--range.first;
			const float& keyframe1 = range.first->first;
			const float& keyframe2 = range.second->first;
			const vertex_cont& vcont1 = range.first->second;
			const vertex_cont& vcont2 = range.second->second;
			for(unsigned int i=0; i<num_vertex; ++i) {
				vector3 dir = (vcont2[i]-vcont1[i])/(keyframe2-keyframe1);
				t_vertex[i] = vcont1[i] + dir*(t-keyframe1);
			}
		}
	}

	for(unsigned int i=0; i<idx_array.size(); ++i) {
		vertex_array[i*3+0] = t_vertex[idx_array[i]].x;
		vertex_array[i*3+1] = t_vertex[idx_array[i]].y;
		vertex_array[i*3+2] = t_vertex[idx_array[i]].z;
	}


	const float *vertex_pointer = &vertex_array[0];
	glVertexPointer(3, GL_FLOAT, 0, vertex_pointer);
	glTexCoordPointer(2, GL_FLOAT, 0, vertex_pointer+idx_array.size()*3 );

	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glDrawArrays(GL_TRIANGLES, 0, idx_array.size());
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_VERTEX_ARRAY);
}

} // gl
} // ist

