#include <stdio.h>
#include <set>
#include <algorithm>
#include "base_polygon_model.h"

#define MAX_BUF 1024

using namespace std;



namespace ist {


BoundingBox BoundingBox::getAABB(const matrix44 &trans) const
{
	BoundingBox aabb;

	for(int i=0; i<8; ++i) {
		vector4 pos;

		switch(i) {
		case 0: pos=min; break;
		case 1: pos.x=max.x; pos.y=min.y; pos.z=min.z; break;
		case 2: pos.x=min.x; pos.y=max.y; pos.z=min.z; break;
		case 3: pos.x=min.x; pos.y=min.y; pos.z=max.z; break;
		case 4: pos=max; break;
		case 5: pos.x=min.x; pos.y=max.y; pos.z=max.z; break;
		case 6: pos.x=max.x; pos.y=min.y; pos.z=max.z; break;
		case 7: pos.x=max.x; pos.y=max.y; pos.z=min.z; break;
		default: break;
		}

		pos=trans*pos;

		if(i==0)
			aabb.max=aabb.min=pos;

		if		(pos.x>aabb.max.x) aabb.max.x=pos.x;
		else if	(pos.x<aabb.min.x) aabb.min.x=pos.x;

		if		(pos.y>aabb.max.y) aabb.max.y=pos.y;
		else if	(pos.y<aabb.min.y) aabb.min.y=pos.y;

		if		(pos.z>aabb.max.z) aabb.max.z=pos.z;
		else if	(pos.z<aabb.min.z) aabb.min.z=pos.z;
	}

	return aabb;
}

bool BoundingBox::overlap(const BoundingBox& bb) const
{
	if(		(max.x < bb.min.x || min.x < bb.max.x)
		&&	(max.y < bb.min.y || min.y < bb.max.y)
		&&	(max.z < bb.min.z || min.z < bb.max.z) )
	{
		return true;
	}
	else {
		return false;
	}
}

bool BoundingBox::inner_point(const BoundingSphere& sphere) const
{
	return inner_point(sphere.pos, sphere.radius);
}

bool BoundingBox::inner_point(const vector4& pos, float radius) const
{
	if(		pos.x < max.x + radius
		&&	pos.x > min.x - radius
		&&	pos.y < max.y + radius
		&&	pos.y > min.y - radius
		&&	pos.z < max.z + radius
		&&	pos.z > min.z - radius )
	{
		return true;
	}
	else {
		return false;
	}
}

int BoundingBox::inner_triangle(const vector4& v1, const vector4& v2, const vector4& v3) const
{
	int result=0;

	if(		min.x < v1.x && max.x > v1.x
		&&	min.y < v1.y && max.y > v1.y
		&&	min.z < v1.z && max.z > v1.z ) ++result;

	if(		min.x < v2.x && max.x > v2.x
		&&	min.y < v2.y && max.y > v2.y
		&&	min.z < v2.z && max.z > v2.z ) ++result;

	if(		min.x < v3.x && max.x > v3.x
		&&	min.y < v3.y && max.y > v3.y
		&&	min.z < v3.z && max.z > v3.z ) ++result;

	return result;
}





BasePolygonChunc::BasePolygonChunc()
	: facet(0)
{}

BasePolygonChunc::~BasePolygonChunc()
{}

void BasePolygonChunc::clear()
{
	vtx_array.clear();
	nml_array.clear();
	tex_array.clear();
	idx_array.clear();
	vnml_array.clear();
	line_array.clear();
	label.clear();
	bounding_box = BoundingBox();
	bounding_sphere = BoundingSphere();
	facet = 0;
}

void BasePolygonChunc::draw() const
{}

void BasePolygonChunc::setLines()
{
	line_array.clear();

	for(unsigned int i=0; i<getFaceNum(); ++i) {
		LineIndex l[3]= {
			LineIndex(idx_array[i*3+0], idx_array[i*3+1]),
			LineIndex(idx_array[i*3+1], idx_array[i*3+2]),
			LineIndex(idx_array[i*3+2], idx_array[i*3+0])
		};

		for(int j=0; j<3; ++j)
			const_cast<LineIndex&>(*line_array.insert(l[j]).first).setFace(i, &idx_array[i*3]);
	}
}


void BasePolygonChunc::setNormal()
{
	if(nml_array.empty()) {
		for(unsigned int i=0; i<getFaceNum(); ++i) {
			vector4 vec1 = vtx_array[ idx_array[i*3+1] ] - vtx_array[ idx_array[i*3] ];
			vector4 vec2 = vtx_array[ idx_array[i*3+2] ] - vtx_array[ idx_array[i*3] ];

			vector4 cur_normal=vector4().cross(vec1, vec2);
			nml_array.push_back(cur_normal);
		}
	}

	if(vnml_array.empty()) {
		for(unsigned int i=0; i<idx_array.size(); ++i) {
			int union_num=0;
			vector4 std_normal=nml_array[i/3];
			vector4 cur_normal=std_normal;

			for (unsigned int j=0; j<getFaceNum(); ++j) {
				if( idx_array[i]==idx_array[j*3+0] || idx_array[i]==idx_array[j*3+1] || idx_array[i]==idx_array[j*3+2] ) {
					if( std_normal!=nml_array[j] && std_normal.cos(nml_array[j]) <= facet*radian) {
						cur_normal+=nml_array[j];
						union_num++;
					}
				}
			}

			cur_normal.normalize();
			vnml_array.push_back( vector3(cur_normal.v) );
		}
	}

	for(unsigned int i=0; i<nml_array.size(); ++i) {
		nml_array[i].normalize();
		nml_array[i].w=0;
	}
}

void BasePolygonChunc::setBoundingVolume()
{
	if(vtx_array.empty())
		return;

	vector4 min = vtx_array.front();
	vector4 max = vtx_array.front();

	for(vector<vector4>::iterator p=vtx_array.begin(); p!=vtx_array.end(); ++p) {
		if(p->x > max.x)
			max.x = p->x;
		else if(p->x < min.x)
			min.x = p->x;

		if(p->y > max.y)
			max.y = p->y;
		else if(p->y < min.y)
			min.y = p->y;

		if(p->z > max.z)
			max.z = p->z;
		else if(p->z < min.z)
			min.z = p->z;
	}

	bounding_box.min = min;
	bounding_box.max = max;

	vector4 center = (max+min)/2;
	bounding_sphere.pos = center;
	bounding_sphere.radius = (max-center).norm();
}


void BasePolygonChunc::setup()
{
	setLines();
	setNormal();
	setBoundingVolume();
}

const string& BasePolygonChunc::getTypeString() const
{
	static const string type_str("BasePolygonChunc");
	return type_str;
}









const BasePolygonModel::chunc_ptr BasePolygonModel::operator [](int index) const
{
	if(!empty() && index<size()) {
		const_iterator p = begin();
		advance(p, index);
		return *p;
	}

	return chunc_ptr();
}

const BasePolygonModel::chunc_ptr BasePolygonModel::operator [](const std::string &label) const
{
	for(const_iterator p=begin(); p!=end(); ++p) {
		if((*p)->label==label)
			return *p;
	}

	return chunc_ptr();
}


void BasePolygonModel::draw() const
{
	for(const_iterator p=begin(); p!=end(); ++p)
		(*p)->draw();
}


bool BasePolygonModel::load(const string& filename)
{
	clear();

	m_filename = filename;

	if(    (strncmp(&filename[filename.size()]-4, ".mqo", 4)==0 && loadMQO(filename))
		|| (strncmp(&filename[filename.size()]-4, ".obj", 4)==0 && loadOBJ(filename)) ) {

		setup();
		return true;
	}

	return false;
}


unsigned int BasePolygonModel::getFaceNum() const
{
	unsigned int num=0;
	for(const_iterator p=begin(); p!=end(); ++p)
		num+=(*p)->getFaceNum();

	return num;
}


unsigned int BasePolygonModel::getVertexNum() const
{
	unsigned int num=0;
	for(const_iterator p=begin(); p!=end(); ++p)
		num+=(*p)->getVertexNum();

	return num;
}


void BasePolygonModel::setup()
{
	for(iterator p=begin(); p!=end(); ) {
		if((*p)->vtx_array.empty()) {	// _z񂪋̃`N͏
			erase(p++);
		}
		else {
			(*p)->setup();
			++p;
		}
	}
}



// OBJǂݍݑ

namespace load_obj {
	char buf[MAX_BUF];
	vector<vector4> lo_vtx_array;
	vector<vector3> lo_nml_array;
	vector<vector2> lo_tex_array;
	BasePolygonModel::chunc_ptr cur_chunc;
	int status=0;

	void clear()
	{
		lo_vtx_array.clear();
		lo_nml_array.clear();
		lo_tex_array.clear();
		status=0;
	}

	void assignVertexArray()
	{
		if(cur_chunc->idx_array.empty())
			return;

		int max_index = *max_element( cur_chunc->idx_array.begin(), cur_chunc->idx_array.end() )+1;
		int min_index = *min_element( cur_chunc->idx_array.begin(), cur_chunc->idx_array.end() );

		cur_chunc->vtx_array.assign( lo_vtx_array.begin()+min_index, lo_vtx_array.begin()+max_index );	// KvȒ_Rs[
		for(vector<int>::iterator p=cur_chunc->idx_array.begin(); p!=cur_chunc->idx_array.end(); ++p)
			*p-=min_index;
	}

	int loadIndex(vector<int> &vtx_index, vector<int> &tex_index, vector<int> &nml_index)
	{
		int tvi, tti, tni;
		int ngon=0;

		char *token;
		token = strtok(buf, " ");
		token = strtok(0, " ");

		while( token!=0 ) {
			switch(status) {
			case(1):
				sscanf(token, "%d", &tvi);
				vtx_index.push_back(tvi-1);
				break;

			case(3):
				sscanf(token, "%d/%d", &tvi, &tti);
				vtx_index.push_back(tvi-1);
				tex_index.push_back(tti-1);
				break;

			case(5):
				sscanf(token, "%d/%d", &tvi, &tni);
				vtx_index.push_back(tvi-1);
				nml_index.push_back(tni-1);
				break;

			case(7):
				sscanf(token, "%d/%d/%d", &tvi, &tti, &tni);
				vtx_index.push_back(tvi-1);
				tex_index.push_back(tti-1);
				nml_index.push_back(tni-1);
				break;

			default:
				break;
			}

			token = strtok( 0, " ");
			++ngon;
		}

		return ngon;
	}

	void loadFace()
	{
		if(strncmp(buf, "f ", 2)!=0)
			return;

		vector<int> vtx_index, tex_index, nml_index;
		int ngon = loadIndex(vtx_index, tex_index, nml_index);

		for(int i=0; i<ngon-2; i++) {	// S3p|Sɕϊ
			int index[3];
			index[0] = 0;
			index[1] = i+1;
			index[2] = i+2;

			for(int j=0; j<3; ++j) {
				if(status&1)
					cur_chunc->idx_array.push_back(vtx_index[ index[j] ]);

				if(status&2)
					cur_chunc->tex_array.push_back( lo_tex_array[ tex_index[ index[j] ] ]);

				if(status&4)
					cur_chunc->vnml_array.push_back( lo_nml_array[ nml_index[ index[j] ] ]);
			}
		}
	}

	int loadVertex()
	{
		if(buf[0]!='v')
			return 0;

		vector4 vtx;
		vector3 nml;
		vector2 tex;

		if( sscanf(buf, "v %f %f %f", &vtx.x, &vtx.y, &vtx.z)==3 ) {
			lo_vtx_array.push_back(vtx);
			return 1;
		}
		else if( sscanf(buf, "vt %f %f", &tex.x, &tex.y)==2 ) {
			lo_tex_array.push_back(tex);
			return 2;
		}
		else if( sscanf(buf, "vn %f %f %f", &nml.x, &nml.y, &nml.z)==3 ) {
			lo_nml_array.push_back(nml);
			return 4;
		}
		else {
			return 0;
		}
	}
}


bool BasePolygonModel::loadOBJ(const std::string& filename)
{
	FILE *fp_obj = fopen(filename.c_str(), "r");
	if(fp_obj==0)
		return false;

	chunc_ptr cur_chunc;
	while (fgets(load_obj::buf, MAX_BUF, fp_obj)!=0) {
		if( strncmp(load_obj::buf, "g ", 2)==0 || !cur_chunc) {
			cur_chunc = createChunc();
			addChunc(cur_chunc);
			load_obj::cur_chunc = cur_chunc;

			char label[128];
			if(sscanf(load_obj::buf, "g %s", label)==1)
				cur_chunc->label = label;

		   	continue;
		}

		int status = load_obj::loadVertex();

		if(status!=0)
			load_obj::status |= status;
		else
			load_obj::loadFace();
	}
	fclose(fp_obj);

	for(iterator p=begin(); p!=end(); ++p) {
		load_obj::cur_chunc = *p;
		load_obj::assignVertexArray();
	}
	for(iterator p=begin(); p!=end(); ) {
		if((*p)->vtx_array.empty())
			erase(p++);
		else
			++p;
	}

	load_obj::clear();

	return true;

}


// MQOǂݍݑ

namespace load_mqo {
	char buf[MAX_BUF];
	FILE *fp_mqo;
	BasePolygonModel::chunc_ptr cur_chunc;

	void loadFace()
	{
		if ( strncmp(buf, "\tface ", 6)!=0 )
			return;

		while( fgets(buf, MAX_BUF, fp_mqo)!=0 && buf[1]!='}') {
			int ngon=0;
			int vtx_index[4];
			vector2 tex[4];

			char *token=strtok(buf, " ");
			sscanf(token, " %d", &ngon);

			while( token!=0 ) {
				if( sscanf(token, " V(%d %d %d %d", &vtx_index[0], &vtx_index[1], &vtx_index[2], &vtx_index[3])==ngon ) {
					for(int i=0; i<ngon-2; ++i) {
						cur_chunc->idx_array.push_back( vtx_index[0] );
						cur_chunc->idx_array.push_back( vtx_index[i+2] );
						cur_chunc->idx_array.push_back( vtx_index[i+1] );
					}
				}
				else if( sscanf(token, " UV(%f %f %f %f %f %f %f %f",
						 &tex[0].x,&tex[0].y, &tex[1].x,&tex[1].y, &tex[2].x,&tex[2].y, &tex[3].x,&tex[3].y)==ngon*2 ) {
					for(int i=0; i<ngon-2; ++i) {
						cur_chunc->tex_array.push_back( tex[0] );
						cur_chunc->tex_array.push_back( tex[i+2] );
						cur_chunc->tex_array.push_back( tex[i+1] );
					}
				}
				token = strtok( 0, ")");
			}
		}
	}

	void loadVertex()
	{
		if ( strncmp(buf, "\tvertex ", 8)!=0 )
			return;

		while(1) {
			vector4 vtx;
			if(	   fgets(buf, MAX_BUF, fp_mqo)!=0
				&& sscanf(buf, "\t\t%f %f %f", &vtx.x, &vtx.y, &vtx.z)==3) {
				cur_chunc->vtx_array.push_back(vtx);
			}
			else {
				break;
			}
		}
	}

	void loadAttrib()
	{
		float facet_;

		if ( sscanf(buf, "\tfacet %f", &facet_)==1 )
			cur_chunc->facet = facet_;
	}

}


bool BasePolygonModel::loadMQO(const std::string& filename)
{
	load_mqo::fp_mqo = fopen(filename.c_str(), "r");
	if(load_mqo::fp_mqo==0)
		return false;

	char t_label[128];
	chunc_ptr cur_chunc;

	while( fgets(load_mqo::buf, MAX_BUF, load_mqo::fp_mqo)!=0) {
		if( sscanf(load_mqo::buf, "Object \"%[^\"]\" {", t_label)==1 ) {
			cur_chunc = createChunc();
			addChunc(cur_chunc);
			cur_chunc->label = t_label;
			load_mqo::cur_chunc = cur_chunc;

			while( fgets(load_mqo::buf, MAX_BUF, load_mqo::fp_mqo)!=0 && load_mqo::buf[0]!='}') {
				load_mqo::loadAttrib();
				load_mqo::loadVertex();
				load_mqo::loadFace();
			}
		}
	}
	fclose(load_mqo::fp_mqo);
	load_mqo::cur_chunc.reset();

	return true;
}



}	// ist



