#include <stdio.h>
#include <stdlib.h>
#include "ist_fw.h"





#ifdef USE_WINAPI


#include <memory>
#include <list>
#include <vector>
#include <utility>
#include <string.h>
using namespace std;


namespace ist {
namespace fw {

HINSTANCE hinst;
MSG  msg;
unsigned int window_message=0;
int joyid=JOYSTICKID1;
JOYINFO joyinfo;




static void InitTimer() {
	TIMECAPS tc;
	::timeGetDevCaps(&tc, sizeof(TIMECAPS));
	::timeBeginPeriod(tc.wPeriodMin);
}


class APIInitializerBase {
public:
	APIInitializerBase() {}
	virtual ~APIInitializerBase() {}

	virtual bool failed()=0;
};

class OpenGLInitializer :  public APIInitializerBase {
public:
	OpenGLInitializer(HWND _hwnd, HDC _hdc) : _failed(false) {
		hwnd=_hwnd;
		hdc=_hdc;

		int color_depth=::GetDeviceCaps(hdc, BITSPIXEL);

		PIXELFORMATDESCRIPTOR pfd;
		memset(&pfd, 0, sizeof(pfd));
		pfd.nSize = sizeof(pfd);
		pfd.nVersion = 1;
		pfd.dwFlags = (PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER);
		pfd.iPixelType = PFD_TYPE_RGBA;
		pfd.cColorBits = color_depth;
		pfd.cRedBits = 5;
		pfd.cGreenBits = 5;
		pfd.cBlueBits = 5;
		pfd.cAlphaBits = (color_depth==16 ? 0 : 8);
		pfd.cAccumRedBits = 0;
		pfd.cAccumGreenBits = 0;
		pfd.cAccumBlueBits = 0;
		pfd.cAccumAlphaBits = 0;
		pfd.cAccumBits = (pfd.cAccumRedBits + pfd.cAccumGreenBits + pfd.cAccumBlueBits + pfd.cAccumAlphaBits);
		pfd.cDepthBits = 16;
		pfd.cStencilBits = 16;

		if(hwnd!=0 && hdc!=0) {
			if(::SetPixelFormat(hdc, ::ChoosePixelFormat(hdc, &pfd), &pfd)==FALSE)
				_failed=true;

			::wglMakeCurrent(hdc, ::wglCreateContext(hdc));
		}
		else {
			_failed=true;
		}
	}

	~OpenGLInitializer() {
		if(hwnd!=0 && hdc!=0)
			ReleaseDC(hwnd, hdc);
	}

	bool failed() {
		return _failed;
	}

private:
	bool _failed;
	HWND hwnd;
	HDC hdc;
};


class WindowCreatorBase
{
public:
	WindowCreatorBase() {}
	virtual ~WindowCreatorBase() {}

	HWND getWindow() const { return hwnd; }
	HDC getDeviceContext() const { return hdc; }

protected:
	void createWindow(HINSTANCE hinst, int width, int height, int flag) {
		WNDCLASS wndclass;
		wndclass.style			= CS_HREDRAW | CS_VREDRAW;
		wndclass.lpfnWndProc	= WndProc;
		wndclass.cbClsExtra		= 0;
		wndclass.cbWndExtra		= 0;
		wndclass.hInstance		= hinst;
		wndclass.hIcon			= NULL;
		wndclass.hCursor		= ::LoadCursor(NULL, IDC_ARROW);
		wndclass.hbrBackground	= static_cast<HBRUSH>(::GetStockObject(BLACK_BRUSH));
		wndclass.lpszMenuName	= NULL;
		wndclass.lpszClassName	= "ist_fw_win";

		::RegisterClass(&wndclass);

		hwnd=::CreateWindow ("ist_fw_win", " ",
					WS_CAPTION | WS_SYSMENU,
					CW_USEDEFAULT,
					CW_USEDEFAULT,
					width, height,
					NULL, NULL, hinst, NULL);

		if(hwnd==0)
			error("EBhE̍쐬Ɏs܂");

		hdc = ::GetDC(hwnd);
		initAPI(flag);
	}

	void error(const char *message) {
		::MessageBox(NULL, message, "Error", 0);
		exit(1);
	}

	typedef auto_ptr<APIInitializerBase> api_initializer;
	api_initializer api_init;
	HWND hwnd;
	HDC hdc;


private:

	void initAPI(int flag) {
		if(flag & WM_OPENGL) {
			api_init=api_initializer(new OpenGLInitializer(hwnd, hdc));
			if(api_init->failed())
				error("OpenGL̏Ɏs܂");
		}
	//	else if(flag & WM_DIRECTX) {}
	
	}


	static LRESULT CALLBACK WndProc (HWND hw, UINT wm, WPARAM wparam, LPARAM lparam) {
		switch(wm) {
		case WM_DESTROY:
			window_message=wm;
			break;
		}

		return ::DefWindowProc(hw, wm, wparam, lparam);
	}

};



class WindowCreatorNormal : public WindowCreatorBase
{
public:
	WindowCreatorNormal(HINSTANCE hinst, int width, int height, int flag) {

		int _width = width + GetSystemMetrics(SM_CXEDGE) + GetSystemMetrics(SM_CXBORDER) + GetSystemMetrics(SM_CXDLGFRAME);
		int _height = height + GetSystemMetrics(SM_CYEDGE) + GetSystemMetrics(SM_CYBORDER)
						+ GetSystemMetrics(SM_CYDLGFRAME) + GetSystemMetrics(SM_CYCAPTION);

		createWindow(hinst, _width, _height, flag);

		::ShowWindow(hwnd, SW_SHOW);
		::UpdateWindow(hwnd);
	}
};

class WindowCreatorFullScreen : public WindowCreatorBase
{
public:
	WindowCreatorFullScreen(HINSTANCE hinst, int width, int height, int flag) {
		::EnumWindows(collectWindowPlacement, 0);

		createWindow(hinst, width, height, flag);

		DEVMODE dev_mode;
		dev_mode.dmSize = sizeof(dev_mode);
		dev_mode.dmFields =DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT | DM_DISPLAYFREQUENCY;
		dev_mode.dmBitsPerPel = 32;
		dev_mode.dmPelsWidth = width;
		dev_mode.dmPelsHeight = height;
		dev_mode.dmDisplayFrequency = 60;

		if( ::ChangeDisplaySettings(&dev_mode, CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL ) {
			error("tXN[Ɏs܂");
		}
		else {
			::Sleep(50);
			::ShowCursor(false);
			::SetWindowLong(hwnd, GWL_STYLE, WS_POPUP);
			::SetMenu(hwnd, NULL);
			::SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, width, height, SWP_SHOWWINDOW);

			::ShowWindow(hwnd, SW_MAXIMIZE);
			::UpdateWindow(hwnd);
		}
	}

	~WindowCreatorFullScreen() {
		::ChangeDisplaySettings(NULL, 0);
		::Sleep(50);
		::EnumWindows(restoreWindowPlacement, 0);
	}


	typedef pair<HWND, WINDOWPLACEMENT> window_attr;
	typedef list< window_attr > window_list;

private:
	static window_list wl;

	static  BOOL CALLBACK collectWindowPlacement(HWND _hwnd, LPARAM lparam)
	{
		if(::IsWindowVisible(_hwnd)){
			wl.push_back( window_attr(_hwnd, WINDOWPLACEMENT()) );
			WINDOWPLACEMENT *wp=&wl.back().second;
			wp->length=sizeof(WINDOWPLACEMENT);
			::GetWindowPlacement(_hwnd, wp);
		}

		return TRUE;
	}

	static BOOL CALLBACK restoreWindowPlacement(HWND _hwnd, LPARAM lparam)
	{
		for(window_list::iterator p=wl.begin(); p!=wl.end(); ++p) {
			if(p->first==_hwnd)
				::SetWindowPlacement(p->first, &p->second);
		}

		return TRUE;
	}
};

WindowCreatorFullScreen::window_list WindowCreatorFullScreen::wl;






static  WindowCreatorBase *wcp=0;

void Init() {
	InitTimer();
}

void Quit() {
	if(wcp!=0)
		delete wcp;
}

bool OpenWindow(int width, int height, int flag) {
	if(flag & WM_FULLSCREEN) wcp = new WindowCreatorFullScreen(hinst, width, height, flag);
	else wcp = new WindowCreatorNormal(hinst, width, height, flag);

	return true;
}

int ClearEvent() {
	while( ::PeekMessage(&msg,0,0,0,PM_REMOVE) ) {
		::TranslateMessage(&msg);
		::DispatchMessage(&msg);
		if(window_message==WM_DESTROY)
			return 1;
	}
	return 0;
/*
	while( ::PeekMessage(&msg,0,0,0,PM_NOREMOVE) ) {
		if( !::GetMessage(&msg,NULL,0,0))
			break;
		::TranslateMessage(&msg);
		::DispatchMessage(&msg);
	}
*/
}

bool GetKeyState(short key) {
	return ::GetAsyncKeyState(key) < 0;
}

int GetMouseState(int *x, int *y) {
	POINT point;
	if( ::GetCursorPos(&point) ) {
		if(x!=0) *x=point.x;
		if(y!=0) *y=point.y;
	}

	int state=0;
	if ( ::GetKeyState(VK_LBUTTON) < 0 ) state |= 1;
	if ( ::GetKeyState(VK_MBUTTON) < 0 ) state |= 2;
	if ( ::GetKeyState(VK_RBUTTON) < 0 ) state |= 4;
	return state;
}


bool SetJoystickID(int id) {
	if(id<2) {
		joyid=id;
		return true;
	}
	else
		return false;
}

bool GetJoystickButton(int button) {
	if( ::joyGetPos(joyid, &joyinfo)!=JOYERR_NOERROR )
		return false;

	return (joyinfo.wButtons & (1<<button)) !=0;
}

short GetJoystickButton() {
	if( ::joyGetPos(joyid, &joyinfo)!=JOYERR_NOERROR )
		return 0;

	return joyinfo.wButtons;
}

short GetJoystickAxis(int axis) {
	if( ::joyGetPos(joyid, &joyinfo)!=JOYERR_NOERROR )
		return 0;

	short neutral=0x7fff;
	switch(axis) {
	case 0:
		if(joyinfo.wXpos==neutral) return 0;
		else if(joyinfo.wXpos>neutral) return joyinfo.wXpos+neutral;
		else if(joyinfo.wXpos<neutral) return joyinfo.wXpos-neutral;
		break;

	case 1:
		if(joyinfo.wYpos==neutral) return 0;
		else if(joyinfo.wYpos>neutral) return joyinfo.wYpos+neutral;
		else if(joyinfo.wYpos<neutral) return joyinfo.wYpos-neutral;
		break;
	}

	return 0;
}


unsigned int GetTicks() {
	return ::timeGetTime();
}

void SwapBuffers() {
	::SwapBuffers(wcp->getDeviceContext());
}




} // fw
} // ist





#endif
// end of _WIN32








#ifdef USE_SDL

using namespace std;


namespace ist {
namespace fw {

class Joystick {
public:
	Joystick() : joy(0), joy_id(0) {}

	explicit Joystick(int id) {
		open(id);
	}

	~Joystick() {
		close();
	}

	short getAxis(int axis) const {
		return SDL_JoystickGetAxis(joy, axis);
	}

	short getButton(int button) const {
		return SDL_JoystickGetButton(joy, button);
	}

	short getButton() const {
		short state=0;
		for(int i=0; i<SDL_JoystickNumButtons(joy); ++i)
			state |= ((SDL_JoystickGetButton(joy, i)!=0)<<i);
		return state;
	}

	bool open(int id) {
		if(isOpened())
			close();

		if(SDL_NumJoysticks()>id) {
			joy_id=id;
			joy=SDL_JoystickOpen(id);
			return true;
		}
		return false;
	}

	void close() {
		if(isOpened()) {
			SDL_JoystickClose(joy);
			joy=0;
		}
	}

private:
	bool isOpened() const {
		return (joy!=0 && SDL_JoystickOpened(joy_id));
	}

	int joy_id;
	SDL_Joystick *joy;
};


class SDLroot {
public:
	SDLroot() {
		if( SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK )<0 ) { 
			printf("error at initializing SDL: %s.\n", SDL_GetError());
			exit(-1);
		}

		joy.open(0);
	}

	~SDLroot() {
		joy.close();
		SDL_Quit();
	}

	Joystick& getJoystick() { return joy; }

	bool openWindow(int width, int height, int ex_flag) {
		const SDL_VideoInfo *info=SDL_GetVideoInfo();
		int bpp=info->vfmt->BitsPerPixel;

		int flag=0;
		if(ex_flag & WM_OPENGL) {
			flag |= SDL_OPENGL;
			SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
			SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
			SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
			SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
			SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
		}

		if(ex_flag & WM_FULLSCREEN) {
			flag |= SDL_FULLSCREEN;
		}

		if( SDL_SetVideoMode( width, height, bpp, flag ) == 0 ) {
			fprintf( stderr, "error at setting video mode: %s\n", SDL_GetError() );
			exit(-1);
		}
		
		return true;
	}

private:

	Joystick joy;
};






static SDLroot *sdl_root=0;

void Init() {
	sdl_root = new SDLroot();
}

void Quit() {
	if(sdl_root!=0)
		delete sdl_root;
}

bool OpenWindow(int width, int height, int ex_flag) {
	return sdl_root->openWindow(width, height, ex_flag);
}

unsigned int GetTicks() {
	return SDL_GetTicks();
}

bool  GetKeyState(short key) {
	unsigned char *keystate=SDL_GetKeyState(0);
	return keystate[key];
}

bool  SetJoystickID(int id) {
	return sdl_root->getJoystick().open(id);
}

bool  GetJoystickButton(int button) {
	return sdl_root->getJoystick().getButton(button);
}

short GetJoystickButton() {
	return sdl_root->getJoystick().getButton();
}

short GetJoystickAxis(int axis) {
	return sdl_root->getJoystick().getAxis(axis);
}

int   GetMouseState(int *x, int *y) {
	return SDL_GetMouseState(x,y);
}

void SwapBuffers() {
	SDL_GL_SwapBuffers();
}

int ClearEvent() {
	SDL_Event event;
	while( SDL_PollEvent(&event) ) {
		switch( event.type ) {
		case SDL_QUIT:
			return 1;
			break;
		}
	}
	return 0;
}


} // fw
} // ist

// end of SDL
#endif 






namespace ist {
namespace fw {

Frame::Frame()  { Init(); }
Frame::~Frame() { Quit(); }

}
}

