#include <assert.h>
#include "gl_include.h"
#include "ist_lua.h"


namespace ist {
namespace lua {

static int luaglGetFloatv(lua_State *L) {
	assert(lua_islightuserdata(L, 2) && "error at gl.GetFloatv");
	glGetFloatv( static_cast<int>(lua_tonumber(L, 1)), static_cast<float*>(lua_touserdata(L, 2)) );
	return 0;
}

static int luaglBlendFunc(lua_State *L) {
	glBlendFunc( static_cast<int>(lua_tonumber(L, 1)), static_cast<int>(lua_tonumber(L, 2)) );
	return 0;
}
static int luaglLogicOp(lua_State *L) { glLogicOp( static_cast<int>(lua_tonumber(L, 1)) ); return 0; }
static int luaglEnable(lua_State *L) { glEnable( static_cast<int>(lua_tonumber(L, 1)) ); return 0; }
static int luaglDisable(lua_State *L) { glDisable( static_cast<int>(lua_tonumber(L, 1)) ); return 0; }


static int luaglMatrixMode(lua_State *L) { glMatrixMode( static_cast<int>(lua_tonumber(L, 1)) ); return 0; }
static int luaglPushMatrix(lua_State *L) { glPushMatrix(); return 0; }
static int luaglPopMatrix(lua_State *L) { glPopMatrix(); return 0; }
static int luaglMultMatrixf(lua_State *L) {
	assert(lua_islightuserdata(L, 1) && "error at gl.MultMatrixf");
	glMultMatrixf( static_cast<float*>(lua_touserdata(L, 1)) );
	return 0;
}
static int luaglLoadMatrixf(lua_State *L) {
	assert(lua_islightuserdata(L, 1) && "error at gl.LoadMatrixf");
	glLoadMatrixf( static_cast<float*>(lua_touserdata(L, 1)) );
	return 0;
}
static int luaglTranslatef(lua_State *L) { glTranslated(lua_tonumber(L,1), lua_tonumber(L,2), lua_tonumber(L,3)); return 0; }
static int luaglRotatef(lua_State *L) { glRotated(lua_tonumber(L,1), lua_tonumber(L,2),lua_tonumber(L,3),lua_tonumber(L,4)); return 0; }
static int luaglScalef(lua_State *L) { glScaled(lua_tonumber(L,1), lua_tonumber(L,2), lua_tonumber(L,3)); return 0; }



static int luaglLightf(lua_State *L) {
	glLightf( static_cast<unsigned int>(lua_tonumber(L, 1)), static_cast<unsigned int>(lua_tonumber(L, 2)),
		static_cast<float>(lua_tonumber(L, 3)) );
	return 0;
}
static int luaglLightfv(lua_State *L) {
	assert(lua_islightuserdata(L, 3) && "error at gl.Lightfv");
	glLightfv( static_cast<unsigned int>(lua_tonumber(L, 1)), static_cast<unsigned int>(lua_tonumber(L, 2)),
		static_cast<float*>(lua_touserdata(L, 3)) );
	return 0;
}

static int luaglMaterialf(lua_State *L) {
	glMaterialf( static_cast<unsigned int>(lua_tonumber(L, 1)), static_cast<unsigned int>(lua_tonumber(L, 2)),
		static_cast<float>(lua_tonumber(L, 3)) );
	return 0;
}
static int luaglMaterialfv(lua_State *L) {
	assert(lua_islightuserdata(L, 3) && "error at gl.Materialfv");
	glMaterialfv( static_cast<unsigned int>(lua_tonumber(L, 1)), static_cast<unsigned int>(lua_tonumber(L, 2)),
		static_cast<float*>(lua_touserdata(L, 3)) );
	return 0;
}


static int luaglTexEnvi(lua_State *L) {
	glTexEnvi( static_cast<unsigned int>(lua_tonumber(L, 1)), static_cast<unsigned int>(lua_tonumber(L, 2)),
		static_cast<int>(lua_tonumber(L, 3)) );
	return 0;
}
static int luaglTexEnvf(lua_State *L) {
	glTexEnvf( static_cast<unsigned int>(lua_tonumber(L, 1)), static_cast<unsigned int>(lua_tonumber(L, 2)),
		static_cast<float>(lua_tonumber(L, 3)) );
	return 0;
}
static int luaglTexEnvfv(lua_State *L) {
	assert(lua_islightuserdata(L, 3) && "error at gl.TexEnvfv");
	glTexEnvfv( static_cast<unsigned int>(lua_tonumber(L, 1)), static_cast<unsigned int>(lua_tonumber(L, 2)),
		static_cast<float*>(lua_touserdata(L, 3)) );
	return 0;
}



inline void RegistFunc(lua_State *L, const char *func_name, int (*func)(lua_State*)) {
	lua_pushstring(L, func_name);
	lua_pushcfunction(L, func);
	lua_settable(L, -3);
}

inline void RegistValue(lua_State *L, const char *value_name, double value) {
	lua_pushstring(L, value_name);
	lua_pushnumber(L, value);
	lua_settable(L, -3);
}

void gllibopen(lua_State *L)
{
	lua_pushstring(L, "gl");
	lua_newtable(L);

	// enable/disable param
	RegistValue(L, "GL_BLEND", GL_BLEND);
	RegistValue(L, "GL_LIGHTING", GL_LIGHTING);
	RegistValue(L, "GL_COLOR_LOGIC_OP", GL_COLOR_LOGIC_OP);
	RegistValue(L, "GL_DEPTH_TEST", GL_DEPTH_TEST);
	RegistValue(L, "GL_TEXTURE_2D", GL_TEXTURE_2D);

	// blend param
	RegistValue(L, "GL_ZERO", GL_ZERO);
	RegistValue(L, "GL_ONE", GL_ONE);
	RegistValue(L, "GL_DST_COLOR", GL_DST_COLOR);
	RegistValue(L, "GL_SRC_COLOR", GL_SRC_COLOR);
	RegistValue(L, "GL_ONE_MINUS_DST_COLOR", GL_ONE_MINUS_DST_COLOR);
	RegistValue(L, "GL_ONE_MINUS_SRC_COLOR", GL_ONE_MINUS_SRC_COLOR);
	RegistValue(L, "GL_SRC_ALPHA", GL_SRC_ALPHA);
	RegistValue(L, "GL_ONE_MINUS_SRC_ALPHA", GL_ONE_MINUS_SRC_ALPHA);
	RegistValue(L, "GL_DST_ALPHA", GL_DST_ALPHA);
	RegistValue(L, "GL_ONE_MINUS_DST_ALPHA", GL_ONE_MINUS_DST_ALPHA);
	RegistValue(L, "GL_SRC_ALPHA_SATURATE", GL_SRC_ALPHA_SATURATE);

	// matrix param
	RegistValue(L, "GL_PROJECTION_MATRIX", GL_PROJECTION_MATRIX);
	RegistValue(L, "GL_MODELVIEW_MATRIX", GL_MODELVIEW_MATRIX);
	RegistValue(L, "GL_TEXTURE_MATRIX", GL_TEXTURE_MATRIX);
	RegistValue(L, "GL_PROJECTION", GL_PROJECTION);
	RegistValue(L, "GL_MODELVIEW", GL_MODELVIEW);
	RegistValue(L, "GL_TEXTURE", GL_TEXTURE);

	// logic op param
	RegistValue(L, "GL_CLEAR", GL_CLEAR);
	RegistValue(L, "GL_AND", GL_AND);
	RegistValue(L, "GL_AND_REVERSE", GL_AND_REVERSE);
	RegistValue(L, "GL_COPY", GL_COPY);
	RegistValue(L, "GL_AND_INVERTED", GL_AND_INVERTED);
	RegistValue(L, "GL_NOOP", GL_NOOP);
	RegistValue(L, "GL_XOR", GL_XOR);
	RegistValue(L, "GL_OR", GL_OR);
	RegistValue(L, "GL_NOR", GL_NOR);
	RegistValue(L, "GL_EQUIV", GL_EQUIV);
	RegistValue(L, "GL_INVERT", GL_INVERT);
	RegistValue(L, "GL_OR_REVERSE", GL_OR_REVERSE);
	RegistValue(L, "GL_COPY_INVERTED", GL_COPY_INVERTED);
	RegistValue(L, "GL_OR_INVERTED", GL_OR_INVERTED);
	RegistValue(L, "GL_NAND", GL_NAND);
	RegistValue(L, "GL_SET", GL_SET);

	// material and light param
	RegistValue(L, "GL_FRONT", GL_FRONT);
	RegistValue(L, "GL_BACK", GL_BACK);
	RegistValue(L, "GL_FRONT_AND_BACK", GL_FRONT_AND_BACK);

	RegistValue(L, "GL_LIGHT0", GL_LIGHT0);
	RegistValue(L, "GL_LIGHT1", GL_LIGHT1);
	RegistValue(L, "GL_LIGHT2", GL_LIGHT2);
	RegistValue(L, "GL_LIGHT3", GL_LIGHT3);
	RegistValue(L, "GL_LIGHT4", GL_LIGHT4);
	RegistValue(L, "GL_LIGHT5", GL_LIGHT5);
	RegistValue(L, "GL_LIGHT6", GL_LIGHT6);
	RegistValue(L, "GL_LIGHT7", GL_LIGHT7);

	RegistValue(L, "GL_AMBIENT", GL_AMBIENT);
	RegistValue(L, "GL_DIFFUSE", GL_DIFFUSE);
	RegistValue(L, "GL_AMBIENT_AND_DIFFUSE", GL_AMBIENT_AND_DIFFUSE);
	RegistValue(L, "GL_SPECULAR", GL_SPECULAR);
	RegistValue(L, "GL_SHININESS", GL_SHININESS);
	RegistValue(L, "GL_POSITION", GL_POSITION);

	// texture param
	RegistValue(L, "GL_TEXTURE_ENV", GL_TEXTURE_ENV);
	RegistValue(L, "GL_TEXTURE_ENV_MODE", GL_TEXTURE_ENV_MODE);
	RegistValue(L, "GL_TEXTURE_ENV_COLOR", GL_TEXTURE_ENV_COLOR);

	RegistValue(L, "GL_MODULATE", GL_MODULATE);
	RegistValue(L, "GL_REPLACE", GL_REPLACE);
	RegistValue(L, "GL_DECAL", GL_DECAL);

	RegistValue(L, "GL_COLOR_INDEX", GL_COLOR_INDEX);
	RegistValue(L, "GL_STENCIL_INDEX", GL_STENCIL_INDEX);
	RegistValue(L, "GL_DEPTH_COMPONENT", GL_DEPTH_COMPONENT);
	RegistValue(L, "GL_RED", GL_RED);
	RegistValue(L, "GL_GREEN", GL_GREEN);
	RegistValue(L, "GL_BLUE", GL_BLUE);
	RegistValue(L, "GL_ALPHA", GL_ALPHA);
	RegistValue(L, "GL_RGB", GL_RGB);
	RegistValue(L, "GL_RGBA", GL_RGBA);
	RegistValue(L, "GL_LUMINANCE", GL_LUMINANCE);
	RegistValue(L, "GL_LUMINANCE_ALPHA", GL_LUMINANCE_ALPHA);


	RegistFunc(L, "GetFloatv", luaglGetFloatv);

	RegistFunc(L, "Enable", luaglEnable);
	RegistFunc(L, "Disable", luaglDisable);
	RegistFunc(L, "BlendFunc", luaglBlendFunc);
	RegistFunc(L, "LogicOp", luaglLogicOp);

	RegistFunc(L, "MatrixMode", luaglMatrixMode);
	RegistFunc(L, "PushMatrix", luaglPushMatrix);
	RegistFunc(L, "PopMatrix", luaglPopMatrix);
	RegistFunc(L, "MultMatrixf", luaglMultMatrixf);
	RegistFunc(L, "LoadMatrixf", luaglLoadMatrixf);
	RegistFunc(L, "Translatef", luaglTranslatef);
	RegistFunc(L, "Rotatef", luaglRotatef);
	RegistFunc(L, "Scalef", luaglScalef);

	RegistFunc(L, "Materialf", luaglMaterialf);
	RegistFunc(L, "Materialfv", luaglMaterialfv);
	RegistFunc(L, "Lightf", luaglLightf);
	RegistFunc(L, "Lightfv", luaglLightfv);

	RegistFunc(L, "TexEnvi", luaglTexEnvi);

	RegistFunc(L, "TexEnvf", luaglTexEnvf);

	RegistFunc(L, "TexEnvfv", luaglTexEnvfv);

	lua_settable(L, LUA_GLOBALSINDEX);

}

} // lua
} // ist

